//
// See the file LICENSE for redistribution information.
//
// Copyright (c) 2002-2004
//	Sleepycat Software.  All rights reserved.
//
// $Id: RenameCommand.cpp,v 1.4 2004/10/18 22:52:13 gmf Exp $
//

#include "RenameCommand.hpp"

using namespace DbXml;
using namespace std;

string RenameCommand::getCommandName() const
{
  return "rename";
}

string RenameCommand::getBriefHelp() const
{
  return "Rename nodes from the documents in the last results returned";
}

string RenameCommand::getMoreHelp() const
{
  return
    string("Takes the path to the nodes to rename, and the new name as\n") +
    string("arguments. The results of the modifications are left in the\n") +
    string("default results. This command creates an XmlModify object using\n") +
    string("XmlManager::createModify(), then adds a rename step to it using\n") +
    string("XmlModify::addRenameStep(). It then executes using\n") +
    string("XmlModify::execute().");
}

void RenameCommand::execute(Args &args, Environment &env)
{
  if(args.size() != 3) {
    throw CommandException("Wrong number of arguments");
  }
  env.testResults();

  if(env.verbose()) cout << "Renaming nodes: " << args[1] << " to: " << args[2] << endl;

  XmlModify modify = env.db().createModify();
  env.results()->reset();

  unsigned int modifications;
  if(env.txn()) {
	  XmlTransaction myTxn = env.childTransaction();
	  modify.addRenameStep(env.db().prepare(myTxn, args[1], env.context()), args[2]);
	  modifications = modify.execute(myTxn, *env.results(), env.context(), env.uc());
	  myTxn.commit();
  }
  else {
	  modify.addRenameStep(env.db().prepare(args[1], env.context()), args[2]);
	  modifications = modify.execute(*env.results(), env.context(), env.uc());
  }

  if(env.verbose()) cout << modifications << " modifications made." << endl;
}

